
 ;  Lawson Labs, Inc.
 ;  model 135 burst mode code
 ;  NOVEMBER 03, 1992
 ;
 ; SAMPLE% MUST BE IN DGROUP !!!!!!!!!!
 ; index values for sample% are in HEX, to match the code
 ; sample%(1) through sample%(7E)  Each scan is of 1 to 64 points. The
 ;            sequence of channels & control codes are stored here.
 ; sample%(80) the first control code (sample%(1)) is repeated after the
 ;             last. For 64-channel scans, that goes here.
 ; sample%(82) A/D Converter address code (ADC) determined by DIP switches
 ; sample%(84) number of bits resolution (8 or 16)
 ; sample%(86) Interrupt flag 1=interrupts on during sampling, 0=off
 ; sample%(88) Starting memory segment for data table
 ; sample%(8A) scans/second (nominal)
 ; sample%(8C) Interchannel delay for high impedance voltage sources
 ; sample%(8E) Number of readings to average for each datapoint
 ; sample%(90) The power of 2 of sample%(8E)
 ; sample%(92) Interscan delay
 ; sample%(94) TICK in nanoseconds (smallest unit for delay loop)
 ; sample%(96) Final ES: ie the segment address one byte past the end of the
 ;             data table
 ; sample%(98) Final SI: ie the displacement into the final segment
 ; sample%(9A) Number of data points per scan * 2
 ; sample%(9C) Int(number of scans/512)
 ; sample%(9E) remainder of scans from sample%(9C) * 512
 ; sample%(A0) DIGITAL INPUT MASK 80H = SCAN STARTED FROM DIGITAL INPUT
 ; sample%(A2) STORAGE FLAG 1 = SAVE DATA
 ; sample%(A4) segment address one byte past last data point actually stored
 ; sample%(A6) offset address for sample%(A6) (used with above to determine if
 ;             sampling was cut short)
 ; sample%(A8) MAXLINE, address of 1st pixel on bottom display line
 ; sample%(AA) & (AC) SCAN DELAY LO & HI WORDS
 ; sample%(AE) TIMING FLAG 0 = NOT TIMING, 1 = LO TIMING, ELSE HI TIMING
                .MODEL MEDIUM
     
                .DATA
SCANELPSLO      DW      ?
SCANELPSHI      DW      ?               ; SCAN DELAY LO & HI WORDS
SCANERROR       DW      0               ; 1 = MISSED DATA
LASTCOUNT       DW      ?               ; LAST COUNTER VALUE

FIRSTCHK        DB      ?               ; FLAG FOR SCAN DELAY ERROR
OVERRUN         DW      ?               ; EXCESS COUNTS FROM SCAN

                .CODE
PUBLIC BURST1, BURSTS, BURSTS2, READCNT100

BURST1      PROC                    ; model 135 1 channel, no delay, no avg
            PUSH    BP
            MOV     BP, SP
            MOV     BX,[BP+06H]     ; BX = SAMPLE% array address
            XOR     SI,SI           ; zero index into array
            MOV     DX,[BX+0082H]   ; get ADC address
            MOV     AL,[BX]         ;       
            OUT     DX,AL           ; output 1st control code
            INC     DX
            IN      AX,DX           ; start conversion
            CALL    SETIMER
            MOV     DX,[BX+0082H]   ; get ADC address
            INC     DX
            MOV     ES,[BX+0088H]   ; segment for data array  
            MOV     AL,[BX+0086H]   ; 1 = interrupts ON, 0 = OFF
            CMP     AL, 1
            JZ      SKIPCLEAR1      ; interrupts ON      
            CLI
SKIPCLEAR1: CALL    READCNT         ; dummy read of counter
            XOR     AX, AX
            MOV     SCANELPSLO, AX
            MOV     SCANELPSHI, AX
            MOV     SCANERROR, AX   ; flag no missed data
            MOV     BP,[BX+0AEH]    ; get timing flag
ZERODP1:    CMP     BP, 0
            JE      NEXTPOINT1      ; not timing
            CALL    READCNT         ; timer update
NEXTPOINT1: IN      AX,DX
            XOR     AH,80H          ; convert from 2's complement
            MOV     ES: [SI],AX     ; place data in data array
            MOV     AX, [BX+00ACH]
            CMP     AX,0
            JE      ONEDELAY        ; no hi byte delay
            MOV     CX,0FFFFH
SCANDEL3:   DEC     CX
            JNZ     SCANDEL3
            PUSH    AX
            CALL    READCNT         ; make sure counter does not roll over
            POP     AX
            DEC     AX
            JNE     SCANDEL3        ; still more hi byte delays
ONEDELAY:   MOV     CX, [BX+00AAH]  ; get lo byte delay
SCANDEL2:   DEC     CX
            JNZ     SCANDEL2
            INC     SI              ; set index to next element
            INC     SI
            JNZ     SAMESEG1        ; data is in the same segment
            MOV     CX,ES           ; set ES to next segment
            ADD     CX,1000H
            MOV     ES, CX
SAMESEG1:   MOV     AX,ES           ; check for last scan
            CMP     AX,[BX+0096H]   ;
            JNZ     ZERODP1         ; not last scan   
            CMP     SI,[BX+0098H]           
            JNZ     ZERODP1         ; not last scan
            JMP     EXIT            ; burst is over
BURST1      ENDP
                               
                                  
BURSTS2     PROC                    ; model 135, multiple channels, no delay
            PUSH    BP              ; no averaging
            MOV     BP, SP
            MOV     BX,[BP+06H]     ; BX = SAMPLE% array address
            XOR     SI,SI           ; zero index into array
            MOV     DX,[BX+0082H]   ; get ADC address
            INC     DX
            IN      AX,DX           ; start conversion
            CALL    SETIMER
            MOV     DX,[BX+0082H]   ; get ADC address
            PUSH    DX
            INC     DX              ; point to data
            MOV     AL,[BX+2]
            OUT     DX,AL
            XOR     DI,DI
            MOV     CL,[BX+2]         ;get 2nd control code
DOSAMPLE:   INC     DI
            INC     DI
            CMP     DI, [BX+009AH]  ;number of data points * 2
            JNZ     DOSAMPLE        ;get to last sample
            INC     DI
            INC     DI
            MOV     [BX+DI],CL      ;put 2nd control code in array
            MOV     ES,[BX+0088H]   ; segment for data array  
            MOV     AL,[BX+0086H]   ; 1 = interrupts ON, 0 = OFF
            CMP     AL, 1
            JZ      SKIPCLEARS      ; interrrupts ON    
            CLI
SKIPCLEARS: CALL    READCNT         ; dummy read of timer
            XOR     AX, AX
            MOV     SCANELPSLO, AX
            MOV     SCANELPSHI, AX
            MOV     SCANERROR, AX   ; flag no missed data
            MOV     BP,[BX+0AEH]    ; get timing flag
ZERODPS:    XOR     DI, DI          ; clear data point counter
CHECKTIM:   CMP     BP, 0
            JE      NEXTPOINTS      ; not timing
            CALL    READCNT         ; timer update
NEXTPOINTS: MOV     AX, [BX+00ACH]
            CMP     AX,0
            JE      ONEDELAY1S      ; no hi byte delay
            MOV     CX,0FFFFH
SCANDEL5S:  DEC     CX
            JNZ     SCANDEL5S
            PUSH    AX
            CALL    READCNT         ; make sure timer does not roll over
            POP     AX
            DEC     AX
            JNE     SCANDEL5S       ; more hi byte delay
ONEDELAY1S: MOV     CX, [BX+00AAH]  ; get lo byte delay
SCANDEL1S:  DEC     CX
            JNZ     SCANDEL1S
            INC     DI
            INC     DI
            MOV     CL,[BX+2+DI]      ;        
            IN      AX, DX
            XCHG    CL,AL
            OUT     DX,AL           ; output NEXT control code
            XCHG    CL,AL
            XOR     AH,80H          ; convert from 2's complement
            MOV     ES: [SI],AX     ; place data in data array
            INC     SI
            INC     SI
            JNZ     SAMESEGS        ; data is in the same segment
            MOV     CX,ES           ; set ES to next segment
            ADD     CX,1000H
            MOV     ES, CX
SAMESEGS:   CMP     [BX+009AH],DI   ; scan done ?
            JNZ     NEXTPOINTS      ; no, clear temp data
            MOV     AX,ES           ; yes, check last scan
            CMP     AX,[BX+0096H]   
            JNZ     ZERODPS         ; not last scan   
            CMP     SI,[BX+0098H]           
            JNZ     ZERODPS         ; not last scan
            CALL    READCNT
            POP     AX
            MOV     [BX+082H],AX    ;restore ADC
; save ES & SI for return to flag incomplete scan
EXIT:       MOV     [BX+00A4H],ES   ; save data ending point
            MOV     [BX+00A6H],SI   ; "         "          "
            MOV     AX, SCANELPSLO
            MOV     [BX+00A0H],AX   ; save total burst time
            MOV     AX, SCANELPSHI
            MOV     [BX+00A2H],AX
            PUSH    SS
            POP     ES
RETPROG:    POP     BP
            STI                         ; enable interrupts  
            MOV     AX, SCANERROR
            MOV     [BX+00AEH], AX      ; flag if data missed
            RETF    2
BURSTS2     ENDP                               
                                  
BURSTS      PROC                    ; model 135 delay time or averaging
            PUSH        BP
            MOV         BP, SP                   
            MOV         BX, [BP+06H]     ; BX = SAMPLE% array address
            XOR         SI, SI           ; zero index into array 
            CALL        SETIMER
            MOV         ES, [BX+0088H]   ; segment for data array   
            MOV         AL, [BX+0086H]   ; 1 = interrupts ON, 0 = OFF
            CMP         AL, 01                   
            JZ          SKIPCLEAR        ; interrupt on    
            CLI                          
 SKIPCLEAR: XOR         AX, AX
            MOV         OVERRUN, AX      ; clear scan over run counts
            MOV         SCANERROR, AX    ; flag no missed data
            MOV         DX, [BX+0082H]   ; get ADC address
            MOV         AL, [BX]         ;         
            OUT         DX, AL           ; output 1st control code
            CALL        READCNT          ; dummy read of timer
            INC         DX
            IN          AX,DX            ; start conversion
            DEC         DX               ; point to status bit
            XOR         AX, AX
            MOV         OVERRUN, AX      ; clear scan over run counts
            MOV         SCANERROR, AX    ; flag no missed data
            MOV         BP, [BX+008CH]   ; interchannel delay
ZERODP:     XOR         DI, DI           ; clear data point counter
STARTSCAN:  MOV         AX, OVERRUN
            MOV         SCANELPSLO, AX   ; set scan interval
            XOR         AX, AX
            MOV         OVERRUN, AX
            MOV         SCANELPSHI, AX
 NEXTPOINT: XOR         AX, AX           ; clear temporary data buffer
            MOV         [BX+00A4H], AX   ;
            MOV         [BX+00A6H], AX   ;         
            MOV         CX, [BX+008EH]   ; CX = # of samples to average
;            CMP         BP, 1
;            JNB         LONGDEL          ; THERE IS AN INTERCHANNEL DELAY
WAITSTATUS: IN          AL, DX                  
            AND         AL, 02                  
            JZ          WAITSTATUS       ; wait for status bit
LONGDEL:    CMP         CX, 01           ; if not last sample to average then
            JNZ         SKIPCHANGE       ; skip channel change output
            INC         DI                      
            INC         DI               ; increment datapoint counter
            MOV         AL,[BX+DI]       ; get next control code     
            OUT         DX,AL            ; output the control code
            CALL        READCNT          ; tag start of inter scan delay
SKIPCHANGE: INC         DX               ; point to data LO byte
            IN          AX, DX           ; get DATA 
            XOR         AH, 80H          ; convert from 2's complement
            DEC         DX               ; point to status bit
            CLC                           
            ADC         [BX+00A4H], AX   ; add data point to buffer
            MOV         AX, 0000                
            ADC         [BX+00A6H], AX   ; for averaging     
            LOOP        WAITSTATUS       ; read the card if more points to avg
            MOV         CX, [BX+0090H]   ; CX = # of / 2  for averaging
            MOV         AX, [BX+00A4H]   ; get LO buffer value
            CMP         CX, 0            ; if no averaging   
            JZ          STOREDATA        ; then save the data
            PUSH        DX               ; save DX
            MOV         DX, [BX+00A6H]   ; get HI buffer value
DODIV:      CLC                                              
            RCR         DX, 1            ; divide by 2    
            RCR         AX, 1            ; Hi and LO bytes    
            LOOP        DODIV            ;           
            POP         DX               ; restore DX    
STOREDATA:  MOV         ES: [SI], AX     ; place data in data array
            INC         SI               ; set index to next element
            INC         SI                      
            JNZ         SAMESEG          ; data is in the same segment
            MOV         CX, ES           ; set ES to next segment
            ADD         CX, 1000H
            MOV         ES, CX                   
SAMESEG:    MOV         CX, BP           ; get interchannel delay
            CMP         CX, 0
            JE          NODELCHK         ; no interchannel delay
CHKDEL:     CALL        READCNT
            STC                          ; sec carry
            SUB         CX, AX           ; decrease interchannel delay
            JNB         CHKDEL           ; not done yet
NODELCHK:   INC         DX               
            INC         DX               ; point to hi byte
            IN          AL,DX            ; start a conversion
            DEC         DX
            DEC         DX
            CMP         [BX+009AH], DI   ; scan done ? 
            JNZ         POINTNEXT        ; no, clear temp data
            MOV         AX, ES           ; yes, check last scan
            CMP         AX, [BX+0096H]
            JNZ         NEXTSCAN         ; not last scan    
            CMP         SI,[BX+0098H]            
            JNZ         NEXTSCAN         ; not last scan

EXITPROG:   JMP         EXIT             ; burst is over
POINTNEXT:  JMP         NEXTPOINT        ; do next datapoint

NEXTSCAN:   XOR         AL, AL           ; do inter scan delay
            INC         AL
            MOV         FIRSTCHK, AL     ; flag first scan delay check
            MOV         CX, [BX+00ACH]   ; get hi delay
SCANDELAY:  CALL        READCNT
            STC
            CMP         CX, SCANELPSHI   ; check hi delay
            JE          DELAYLO          ; check lo delay
            JB          FLGCHK
            MOV         FIRSTCHK, 0      ; flag scan delay ok
            JMP         SCANDELAY
DELAYLO:    MOV         CX, [BX+00AAH]   ; get lo delay
            STC
            CMP         CX, SCANELPSLO   ; decrease delay
            JB          SETOVR           ; scan delay complete
            MOV         FIRSTCHK, 0      ; flag scan delay ok
DELAYSCAN:  CALL        READCNT
            STC
            CMP         CX, SCANELPSLO
            JE          FLGCHK
            JNB         DELAYSCAN        ; loop till lo delay done
SETOVR:     MOV         AX, SCANELPSLO          
            SUB         AX, CX
            MOV         OVERRUN, AX
FLGCHK:     CMP         FIRSTCHK, 0
            JE          DELOK            ; scan delay is ok
            INC         SCANERROR        ; else flag missed data
DELOK:      INC         DX               
            INC         DX               ; point to hi byte
            IN          AL,DX            ; start a conversion
            DEC         DX
            DEC         DX
            JMP         ZERODP           ; do next scan
BURSTS  ENDP
                                  

READCNT         PROC NEAR               ; read timer 2
                                        ; uses AX, CX, DX
                PUSH CX
                PUSH DX
                MOV DX, 43H             ; 8254 mode reg
                MOV AL,80H              ; latch timer 2
                OUT DX, AL
                DEC DX                  ; timer 2 count port     
                IN AL, DX               ; get lo byte
                XCHG AL,AH              ; save lo byte
                IN AL, DX               ; get hi byte
                XCHG AL, AH             ; swap hi & lo
                MOV CX, LASTCOUNT
                SUB CX, AX              ; get count difference
                CMP CX, 8000H
                JB POSITIVE
                XOR CX, 0FFFFH          ; correct for negative answer
                INC CX
      POSITIVE: XCHG CX, AX             ; AX = interval, CX = last count
                MOV LASTCOUNT, CX       ; LASTCOUNT = last counter read
                CLC                     ; inc scan delay counter
                ADC SCANELPSLO, AX
                MOV CX, 0
                ADC SCANELPSHI, CX
                POP DX
                POP CX
NOCNT:          RET
READCNT ENDP

SETIMER     PROC NEAR               ; setup timer 2            

            MOV     DX, 43H         ; timer 2 address
            MOV     AL, 0B0H        ; mode 2 code
            OUT     DX, AL
            XOR     AL, AL
            DEC     DL
            OUT     DX, AL
            OUT     DX,AL           ; set max count
            MOV     DX, 61H
            IN      AL, DX
            OR      AL, 1
            AND     AL, 0FDH
            OUT     DX, AL          ; disable sound
            RET
SETIMER ENDP

READCNT100  PROC
            PUSH    BP
            MOV     BP, SP
            MOV     BX,[BP+06H]         ; BX = SAMPLE% array address
            XOR     SI,SI               ; zero index into array
            CALL    SETIMER
            CLI
            CALL    READCNT             ; dummy read
            XOR     AX,AX
            MOV     SCANELPSLO, AX
            MOV     SCANELPSHI, AX
            MOV     CX, [BX+00A2H]      ; loop counter
DOCNT:      CALL    READCNT
            DEC     CX
            JNZ     DOCNT
            MOV     [BX+00AAH],ES       ; save data ending point
            MOV     [BX+00ACH],SI       ; "         "          "
            MOV     AX, SCANELPSLO
            MOV     [BX+00A0H],AX
            MOV     AX, SCANELPSHI
            MOV     [BX+00A2H],AX
            PUSH    SS
            POP     ES
            POP     BP
            STI                         ; enable interrupts   
            RETF    2
READCNT100 ENDP


END
*

